#include <Arduino.h>

/*
Вариант на три исполнительных устройства.

Изменения по оптимизации работы с GPRS.


22.07.2014



http://goo.gl/jbwrl1 - управление GPRS Shield
http://arduino.cc/en/Main/ArduinoBoardMega2560
https://github.com/MarcoMartines/GSM-GPRS-GPS-Shield GSM GPRS Shield, SIM900, IDE 1.00,  Библиотека SIM900

https://github.com/aaronds/arduino-nrf24l01		- nrf24l01
http://playground.arduino.cc/InterfacingWithHardware/Nrf24L01
http://maniacbug.github.io/RF24/examples.html

Мега:	Serial 2:		17 (RX) and 16 (TX);
		PWM:			2 to 13, 44-46;
		bottonInput		10 - кнопка;
		SPI:			50 (MISO), 51 (MOSI), 52 (SCK), 53 (SS);
*/

#include <stdint.h>
#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>

#define CE_PIN  12                          // Ноги для GPRS;
#define CSN_PIN 53
                                            // Определяем transmit pipes
const uint64_t pipes[3] = {0xE8E8F0F0E1LL, 0xE8E8F0F0EELL, 0xE8E8F0F0D0LL};

RF24 radio(CE_PIN, CSN_PIN);                // Создаем класс radio

#define bottonInput 10                      // Нога для кнопки
#define pinGotMovie 2                       // Нога для ИК сенсора
#define pinNight A0						    // Нога для сенсора освещенности;
#define pinToSwitchOnGPRS 9                 // Нога включения GPRS

#define pinShelf 6                          // Нога для родной подсветки полок;
#define pinRed 3                            // Три PWM цвета;
#define pinGreen 4
#define pinBlue 5

#define sensorNight 1020				    // Если выше - то ночь;
#define TIME_TO_GOT_TOILET 11000            // Время длительности включения ночной подсветки;
#define delayNextStepMoodLamp 5             // Скорость изменения лампы настроения;
#define TIME_TO_NEXT_SMS 60000              // Время до отправки следующего СМС сообщения;
#define TIME_TO_SWITCH_SECURITY_MODE 20000
int sensorA = 0, sensorB = 0, sensorNow;    // Счетчики для решения вопроса день/ночь;
uint8_t sensorStep = 0;

uint8_t i = 0;							    // Счетчик смены цветов;
                                            // Для работы с цветами;
uint8_t redOld = 0, greenOld = 0, blueOld = 0, red = 0, green = 0, blue = 0;

String currStr = "";                        // Стринг для чтения СМС;
uint8_t countSMSMessages = 0;               // Считаем отправленные тревожные СМС;

unsigned long timeToStepMoodLamp = 0;       // Переменная для скорости лампы настроения;
unsigned long timeWhenGotImpulse = 0;       // Переменная для отсчета времени нажатия кнопки;
unsigned long timeWhenStartImitation = 0;   // Переменная для отсчета времени режима имитации присутствия;
#define TIME_TO_COUNT_IMITATION 6000000     // Время (сто минут) имитации присутствия;
unsigned long timeWhenDesigionMade = 0;     // Переменная для отсчета времени для постановки на охрану по кнопке;
unsigned long timeWhenGotMovie = 0;         // Переменная времени обнаружения движения;
                                            // Переменная для отсчета времени задержки включения ночного режима;
unsigned long timeWhenSwitchOnForMovie = 0;
unsigned long timeToUpdateGPRS = 0;         // Переменная для поддержания связи с GPRS;

enum modesOfUnitToWork {                    // Режимы работы устройства:
    modeDutyNormal,							// Дежурный;
    modeOn,									// Освещение включено;
    modeOnMoode,							// Лампа настроения;
    modeSecurity,							// Охрана;
} modesOfUnit;

struct flags {                              // Флаги;
    unsigned nightNow      : 1;             // Сейчас ночь;
    unsigned switchOn      : 1;             // Освещение включено;
    unsigned securityOn    : 1;             // Стою на охране;
    unsigned bottonShort   : 1;             // Кнопка: короткое нажатие;
    unsigned moveGot       : 1;             // Обнаружено движение;
    unsigned bottonLong    : 1;             // Кнопка: длинное нажатие;
    unsigned mySMS         : 1;             // Это СМС подлежит обработке;
    unsigned sentSMS       : 1;             // СМС отправлено;
} flag;

enum ProgrammStepsToWork {                  // Шаги программы:;
    stepCheckBotton,                        // Проверить кнопку;
    stepCheckGPRS,                          // Проверить GPRS модуль;
    stepCheckLight,                         // Проверить датчик освещенности;
    stepCheckMove,                          // Проверить датчик движения;
    stepWhatToDo,                           // Принять решение по проверке датчиков;
    stepDutyNormal,                         // Обычный режим;
    stepSwitchON,                           // Включить освещение;
    stepMoodLamp,                           // Лампа настроения;
    stepStartSecurity,                      // Задержка включения охраны;
    stepSecurityOn,                         // Охраняю;
} ProgrammSteps;

enum bottonStepsToWork {                    // Шаги проверки кнопок;
    bottonGotImpulse,                       // Обнаружен импульс на кнопке;
    bottonDelay150,                         // Перепроверка через 150 мс;
    bottonEnd,                              // Решение о продолжительности нажатия кнопки;
} bottonSteps;

//******************************* Объявляем функции: ****************************************//
void SendNRFCommand(int);                   // Отправка команды через nrf24l01;
void SendNRFCimitation(void);               // Отправка команды на имитацию пристутствия через nrf24l01;
void SwitchAllOff (uint8_t);                // Выключить всё - два режима, 0 - сразу, 1 - через полное включение;
void SendSMSString(String);                 // Отправка текста через СМС;
void SetGPRSMode(void);                     // Установка режима работы модуля GPRS;
void CheckGPRS(void);                       // Включение/проверка включения GPRS;
void TouchGPRS(void);                       // Поддержание связи с GPRS;
void DialVoiceCall();                       // Прозвон на установленный номер;
void PWMWrite(uint8_t, uint8_t, uint8_t);   // Запись PWM;



void setup()
{
// 	Serial.begin(19200);

    /************************* Для GPRS **************************/
    Serial2.begin(19200);
    pinMode(pinToSwitchOnGPRS,OUTPUT);      // Для зажигания GPRS;
    pinMode(pinShelf, OUTPUT);              // Нога для включения подсветки полок;
    //SwitchOnGPRS();
    CheckGPRS();                            // Включаем GPRS если он выключен;
    delay(500);
    SetGPRSMode();                          // Устанавливаем режим GPRS;
    delay(500);

    /************************* Модуль NRF24 **********************/
    radio.begin();                          // Включение модуля;
    radio.setChannel(0);                    // Установка канала вещания;
    radio.setRetries(15,15);                // Установка интервала и количества попыток "дозвона" до приемника;
    radio.setDataRate(RF24_250KBPS);        // Установка минимальной скорости;
    radio.setPALevel(RF24_PA_MAX);          // Установка максимальной мощности;
    radio.setAutoAck(1);                    // Установка режима подтверждения приема;

    PWMWrite(255,255,255);                  // Включим подсветку;

    SendNRFCommand(100);                    // Командаы на включение и выключение дистанционных модулей;
    delay (1000);
    SendNRFCommand(101);
    delay (1000);
    SendNRFCommand(100);
 	delay (1000);

    PWMWrite(0,0,0);                        // Выключим подсветку;

    modesOfUnit = modeDutyNormal;           // Режим устройства - обычный;
    ProgrammSteps = stepCheckBotton;        // Программу начинаем с проверки кнопки;
    bottonSteps = bottonGotImpulse;         // Кнопку проверяем с обнаружения нажатия;
    //timeToStepMoodLamp = millis();
}

void loop()
{
    switch (ProgrammSteps)
    {
    /************************* stepCheckBotton **************************/
    case stepCheckBotton:                   // Проверка кнопки;
//        Serial.print("Flag SwitchOn  = ");
//        Serial.println(flag.switchOn);
//        Serial.print("Mode of Unit is ");
//        Serial.println(modesOfUnit);

        switch (bottonSteps) {              // Три шага проверки;

        case bottonGotImpulse:
            if (0 == digitalRead(bottonInput)) {    // Ловим первый импульс нажатия;
                timeWhenGotImpulse = millis();      // Запоминаем время нажатия;
                bottonSteps = bottonDelay150;       // В следующий раз проверка в другом switch(е);
            }
            break;

        case bottonDelay150:                        // Через 150 мс;
            if (millis() > (timeWhenGotImpulse+150)) {
                if (0 == digitalRead(bottonInput)) {// Смотрим, нажаа ли кнопка;
                    bottonSteps = bottonEnd;        // И если нажата - переходим к третему шагу проверки;
                }
                else {
                    bottonSteps = bottonGotImpulse; // Иначе будем проверять кнопку заново;
                }
            }

            break;

        case bottonEnd:
            while (0 == digitalRead(bottonInput))   // Кнопка все еще нажата;
            {                                       // И если нажата более 1,5 с;
                if (millis() > (timeWhenGotImpulse+1500)) {
                    PWMWrite(255,0,0);              // Зажигаем красный цвет;
                }
                else {
                    PWMWrite(0,255,0);              // Но сначала зажигаем зеленый;
                }
            }

            if (millis() > (timeWhenGotImpulse+1500) ) {
                flag.bottonLong = 1;                // Если кнопка более 1,5 с - флаг длинного нажатия;
            }
            else {
                flag.bottonShort = 1;               // Иначе флаг короткого нажатия;
            }
            PWMWrite(0,0,0);                        // Гасим ленту;
            bottonSteps = bottonGotImpulse;         // Идем дальше;
            break;
        }

        ProgrammSteps = stepCheckGPRS;
        break;

    /************************ stepCheckGPRS ***************************/
    case stepCheckGPRS:
//        Serial.println("Step Check GPRS");
        TouchGPRS();                                        // Периодически сигнал на GPRS чтобы не спал;
        if (Serial2.available()) {                          // Если GPRS имеет что сказать,
            char currSymb = Serial2.read();                 // Чиаем его;
            if ('\r' == currSymb) {                         // И если все сказано - сравниваем с эталонами ниже;
//                Serial.println(currStr);
                if (!currStr.compareTo("RING")) {           // Если идет звонок;
                    Serial2.println("AT+CLCC");             // Запрос номера звонящего;
                }
                                                            // Если номер "наш";
                if((currStr.indexOf("71111111111") !=-1) && (currStr.indexOf("CLCC") !=-1)) {
                    delay(3000);                            // Даем услышать звонок;
                    Serial2.println("ATA");		            // Посылаем команду на поднятие трубки;
                }
                                                            // Если пришло СМС с "нашего" номера"
                if(flag.mySMS == 1) {
                    flag.mySMS = 0;                             // Сброс флага;
                    if (!currStr.compareTo("arm")) {            // Если команда постановки на охрану;
                            SwitchAllOff(1);                    // Все выключить через включение;
                            flag.switchOn = 0;                  // и сбросить флаг "включено";
                            modesOfUnit = modeSecurity;         // Режим охраны - установить;
                            flag.securityOn = 1;                // Причем - немедленной охраны;
                            flag.moveGot = 0;
                            flag.sentSMS = 0;                   // Обнуляем флаг отправки СМС;
                            countSMSMessages = 0;               // Обнуляем счетчик отправленных СМС;
                                                                // Оповещаем о постановке на охрану;
                            SendSMSString("Security Started from SMS!");

 //                         Serial.println("Start ARM Now!");
                    }

                   else if (!currStr.compareTo("imit")) {           // Если команда включения имитации;
                        if(modesOfUnit == modeSecurity) {
                                timeWhenStartImitation = millis();  // Установили время начала;
                                SendNRFCimitation();                // Команда имитации присутствия;
                                flag.switchOn = 1;                  // и установить флаг "включено";
                                SendSMSString("Imitation Started!");// Оповещаем о начале имитации;
//                              Serial.println("Start Imitation Now!");
                        }
                        else {
                                SendSMSString("Error: No Security Now! No imitation!");// Оповещаем об ошибке;
                        }

                    }
                  else  if (!currStr.compareTo("noimit")) {         // Если команда выключения имитации;
                            if(modesOfUnit == modeSecurity && flag.switchOn == 1) {
                                SendNRFCommand(100);                // Выключение подсветки;
                                flag.switchOn = 0;                  // и сбросить флаг "включено";
                                SendSMSString("Imitation Stopped!");     // Оповещаем о событии;
                            }
                            else {
                                SendSMSString("Error: There Was Not The Imitation!");// Оповещаем об ошибке;
                            }

 //                         Serial.println("Stop Ivitation Now!");

                    }

                  else  if (!currStr.compareTo("disarm")) {         // По аналогии - снятие с охраны;
                        if(modesOfUnit == modeSecurity) {
                                if (flag.switchOn == 1) {
                                    SendNRFCommand(100);                // Выключение подсветки имитации;
                                    flag.switchOn = 0;                  // и сбросить флаг "включено";
                                }
                                modesOfUnit = modeDutyNormal;
                                flag.securityOn = 0;
                                SendSMSString("Disarmed from SMS!");
  //                            Serial.println("DISARM now!");
                            }
                            else {
                                SendSMSString("Error: There Was Not Security Mode!");
                           }
                    }
                  else if (!currStr.compareTo("mode")) {           // Оповещение о текущем режиме;
                        if(modesOfUnit == modeSecurity) {   // Если охраняем;
                                if(flag.switchOn) {             // И имитируем присутствие;
                                    SendSMSString("Security And Imitation Now!");
                                }
                                else {
                                    SendSMSString("Only Security Mode Now!");
                                }
                            }
                            else {                              // А если не охраняем;
                                SendSMSString("Disarmed Now!");
                            }
                    }

                   else {
                        SendSMSString("Are You Kidding? You Are Wrong!");
                   }

                }


                 if (currStr.indexOf("CMT") !=-1 && currStr.indexOf("71111111111") !=-1 ) {
                    flag.mySMS = 1;                         // Устанавливаем флаг обработки СМС;
  //                  Serial.println("Set Flag");
                }


                if (!currStr.compareTo("NO CARRIER")) {     // При разрыве связи после звонка;
  //                  Serial.println("NO CARRIER - Set GPRS Mode");
                    SetGPRSMode();                          // Восстанавливаем режим приема СМС;
                }

                if (!currStr.compareTo("NORMAL POWER DOWN")){ // При выключении GPRS по какми-то причинам (перезагрузка, например);
                    CheckGPRS();                            // Включаем;
                    SetGPRSMode();                          // Восстанавливаем режим приема СМС;
                }

                currStr = "";
                //Serial2.println("AT+CMGD=1,4\r");         // Удаляем все SMS;
            }
            else if (currSymb != '\n') {                    // В цикле, если не конец СМС,
                currStr += String(currSymb);                // Формируем текст сообщения из порта GPRS;
            }
        }

        ProgrammSteps = stepCheckLight;                     // Идем дальше;
        break;

    /************************ stepCheckLight ***************************/
    case stepCheckLight:                                    // Проверка день/ночь;
        if (modesOfUnit == modeDutyNormal) {                // Только для обычного режима;
            sensorNow = analogRead(pinNight);               // Читаем сенсор;
// 				Serial.print("Light Now = ");
// 				Serial.println(sensorNow);

            if (sensorNow > sensorNight) {
                sensorA++;                                  // Если темнее референсного увеличиваем счетчик;
            }
            if (sensorNow > (sensorNight - 20)) {           // Если темнее (референсного минус гистерезис) увеличиваем другой счетчик;
                sensorB++;
            }
            sensorStep++;                                   // Увеличиваем количество заходов подсчета;

            if (sensorStep == 20) {                         // После 20 решаем день или ночь;
                if (sensorA >18) {                          // Если темнее "ночи" 18 раз из 20
                    flag.nightNow = 1;                      // Флаг - ночь;
                }
                if (sensorB == 0) {                         // Если счетчик за вычетом гистерезиса ни разу не сработал
                    flag.nightNow = 0;                      // Уже - день;
                }
                sensorStep = 0;                             // Обнуляем счетчики;
                sensorA = 0;
                sensorB = 0;
            }
        }

        ProgrammSteps = stepCheckMove;
        break;

        /************************ stepCheckMove ***************************/
    case stepCheckMove:                                     // Проверка движения или ночью или при охране;
        if (((modesOfUnit == modeDutyNormal) && flag.nightNow ) || ((modesOfUnit == modeSecurity) && (flag.securityOn)))
        {
            if (digitalRead(pinGotMovie)) {                 // Читаем сенсор;
                flag.moveGot = 1;                           // Ставим флаг движения;
//                timeWhenGotMovie = millis();                // Запоминаем время захвата движения;
            }
        }
        ProgrammSteps = stepWhatToDo;
        break;

    /************************ stepWhatToDo ***************************/
    case stepWhatToDo:                                      // Включаем мозги на основе изучения ысего что выше;
        if (flag.bottonShort) {                             // Если было короткое нажатие кнопки;
            flag.bottonShort = 0;                           // Сбрасываем флаг нажатия;
                                                            // Включаем следующий режим работы;
            modesOfUnit = (modesOfUnitToWork)(modesOfUnit + 1);
            //SendNRFCommand(100);                            // Гасим все - если нужно потом включим что надо;
            SwitchAllOff(0);
            flag.switchOn = 0;
            if (modesOfUnit >= modeSecurity) {              // Если перебором (короткое нажатие) включится режим охраны (он последний);
                modesOfUnit = modeDutyNormal;               // Возвращаем все в начало;
                flag.securityOn = 0;                        // Сбрасываем флаги, потому что коротким нажатием можно тоже снять с охраны;
                flag.sentSMS = 0;
            }
// 				Serial.print( "modesOfUnit = ");
// 				Serial.println(modesOfUnit);
        }

        if (flag.bottonLong) {                              // Если длинное нажатие;
            flag.bottonLong = 0;                            // Сброс всех флагов;
            flag.securityOn = 0;
            flag.nightNow   = 0;
            flag.switchOn   = 0;
            flag.sentSMS    = 0;
            flag.moveGot    = 0;

            if (modesOfUnit == modeSecurity) {              // Если режим охраны - снимаем;
                modesOfUnit = modeDutyNormal;
            }
            else {                                          // Во всех остальных случаях,
                modesOfUnit = modeSecurity;                 // Запускаем постановку на охрану;
                timeWhenDesigionMade = millis();
                countSMSMessages = 0;
            }

            SwitchAllOff(1);                                // Оповещаем о длинной кнопке выключением через включение;
// 				Serial.print( "modesOfUnit = ");
// 				Serial.println(modesOfUnit);
        }

        ProgrammSteps = stepDutyNormal;
        break;

    /************************ stepDutyNormal ***************************/
    case stepDutyNormal:

        if (modesOfUnit == modeDutyNormal)                  // В обычном режиме;
        {
            //Serial.println("Now Duty Normal");            // Если рбнаружено ночью движение и освещение выключено;
            if (flag.moveGot && flag.nightNow) {
                flag.moveGot = 0;                           // Сброс флага;

                if(!flag.switchOn) {
                    SendNRFCommand(103);                    // Команда на включение подсветки модулям;
                    //Serial.println("Send 103");
                    PWMWrite(15,15,15);                     // Включение подсветки на самой Меге;
                    flag.switchOn = 1;                      // Устапновка флага включения подсветки;
                    timeWhenSwitchOnForMovie = millis();    // Запомнили время включения;
                }
                else {                                      // Если двиежние продолжается, а подсветка включена;
                    flag.moveGot = 0;                       // Флаг движения сбрасываем,
                    timeWhenSwitchOnForMovie = millis();    // А время подсветки перезапоминаем;
                }
            }


            // Выключем подсветку или по истечении времени и отсутствию движения или если включили общий свет и сброшет флаг "ночь";
            if ( (!flag.moveGot && flag.switchOn && (millis() > (timeWhenSwitchOnForMovie + TIME_TO_GOT_TOILET))) || (!flag.nightNow && flag.switchOn) )
            {
                //timeWhenSwitchOnForMovie = millis();
                flag.switchOn = 0;
                SwitchAllOff(0);
            }
        }

        ProgrammSteps = stepSwitchON;
        break;


        /************************ stepSwitchON ***************************/
    case stepSwitchON:
        if (modesOfUnit == modeOn)                          // Если режим "Включено", но флаг "включено" не установлен
        {
            if (!flag.switchOn) {
                SendNRFCommand(101);                        // Включаем модули;
                PWMWrite(255,255,255);                      // Включаем Мегу;
                digitalWrite(pinShelf,1);                   // Включаем полки;
                flag.switchOn = 1;						    // Устанавливаем флаг включения;
             }
        }
        ProgrammSteps = stepMoodLamp;
        break;


        /************************ stepMoodLamp ***************************/
    case stepMoodLamp:
        if (modesOfUnit == modeOnMoode) {                   // Режим лампы настроения;
            if (!flag.switchOn) {
                flag.switchOn = 1;                          // Объявим флаг включения;
                SendNRFCommand(102);                        // Команда исполнительым модулям;
                digitalWrite(pinShelf,1);                   // Включение подсветки полок;
                //Serial.println("Send 102");
            }
            //Serial.println("Now Mood Lamp");
                                                            // С задержкой перебираем цвета;
            if (millis() > timeToStepMoodLamp + delayNextStepMoodLamp) {
                if ((red == redOld) && (green == greenOld) && (blue == blueOld) ) { // Если зажжены целевые цвета;
                    do {                                    // Находим новые случайно;
                        red   = random(255);
                        green = random(255);
                        blue  = random(255);
                    }                                       // Причем чтобы между ними была разница не менее 140;
                    while (((abs)(red - green) < 140) && ((abs)(green - blue) < 140) && ((abs)(blue - red) < 140 ) );

                    i++;			                        // Счетчик смены цветов;
                    if (10 == i) {                          // Периодически устанавливаем чистые цвета;
                        red = 255; green = 0; blue = 0;     // Не доверяя случаю;
                    }
                    if (20 == i) {
                        red = 0; green = 255; blue = 0;
                    }
                    if (30 == i) {
                        red = 0; green = 0; blue = 255;
                    }
                    if (40 == i) {
                        red = 255; green = 255; blue = 0;
                    }
                    if (50 == i) {
                        red = 0; green = 255; blue = 255;
                    }
                    if (60 == i) {
                        red = 255; green = 0; blue = 255;
                    }
                    if (70 == i) {
                        red = 255; green = 255; blue = 255;
                        i = 0;
                    }
                }
                PWMWrite(redOld, greenOld, blueOld);        // Зажигаем цвета;

                if (red > redOld) redOld++;                 // И подтягиваем к целевым следующим шагом;
                if (red < redOld) redOld--;
                if (green > greenOld) greenOld++;
                if (green < greenOld) greenOld--;
                if (blue > blueOld)	blueOld++;
                if (blue < blueOld) blueOld--;

                timeToStepMoodLamp = millis();
            }
        }
        ProgrammSteps = stepStartSecurity;
        break;


        /************************ stepStartSecurity ***************************/
    case stepStartSecurity:                                         // Режим паузы перед режимом охраны;

        if ((modesOfUnit == modeSecurity) && !flag.securityOn) {    // Нужно некоторое время, чтобы включить режим охраны

            if(millis() > (timeWhenDesigionMade + TIME_TO_SWITCH_SECURITY_MODE)) {         // и убежать... 20 секунд задержки.
                flag.securityOn = 1;                                // Выставляем флаг охраны;
                flag.moveGot = 0;
                SendSMSString("Security From Botton!");             // Отправляем СМС о том, что охраняем;
            }
        }

        ProgrammSteps = stepSecurityOn;

        break;

        /************************ stepSecurityOn ***************************/
    case stepSecurityOn:                                            // Режим охраны;
        //Serial.println("stepSecurityOn");
        if ((modesOfUnit == modeSecurity) && flag.securityOn) {     // Работает, когда установлен режим охраны и флаг "охраняю;
        // Когда обнаружено движение и нет флага об отправке СМС - немедленная отправка, иначе каждые 60 сеунд;
                if ((flag.moveGot && !flag.sentSMS) || (flag.moveGot && (millis() > (timeWhenGotMovie + TIME_TO_NEXT_SMS )))) {
                    flag.moveGot = 0;                               // Сброс флага обнаружения движения;
                    timeWhenGotMovie = millis();                    // Запись времени обнаружения движения;
                    if(countSMSMessages < 3) {
                        SendSMSString("ALARM! Got Movement!");     // Шлем СМС.
//                        Serial.println("Send Alarm Message!");
                        flag.sentSMS = 1;                           // Устанавливаем флаг отправки СМС;
                    }

                    if(countSMSMessages == 3) {
//                            Serial.print("Count Message = 3!!!! ");
                            DialVoiceCall();                            // Прозваниваемся, и бросаем эту дурную затею
                    }

                    countSMSMessages++;
//                    Serial.print("Count Message: ");
//                    Serial.println(countSMSMessages);
                }                                                   // Выключим имитацию, если время пришло;
                if ((flag.switchOn == 1) && ((millis() - timeWhenStartImitation) > TIME_TO_COUNT_IMITATION )) {
                      SwitchAllOff(0);
                      flag.switchOn = 0;
                      SendSMSString("Imitation Stopped By Timer!");
                }

        }

        ProgrammSteps = stepCheckBotton;
        break;

    default:
        ProgrammSteps = stepCheckBotton;
        break;
    }
}

void SendNRFCommand(int command) {                                  // Отправка команды модулям;

    for (uint8_t countPipes = 0; countPipes <3; countPipes++ ) {    // Которых две штуки;
        radio.openWritingPipe(pipes[countPipes]);                   // Открываем соответствующую трубу;
        radio.flush_tx();                                           // Сливаем буфер передачи;
        radio.write( &command, sizeof(command) );                   // Отправляем команду;
    }
}

void SendNRFCimitation(void) {                                      // Типа как предыдущая функция;
    int on = 101;                                                   // Только разные команды на разные модули;
    int mood = 102;
    radio.openWritingPipe(pipes[0]);
    radio.flush_tx();
    radio.write( &on, sizeof(on) );
    radio.openWritingPipe(pipes[1]);
    radio.flush_tx();
    radio.write( &mood, sizeof(mood) );
    radio.openWritingPipe(pipes[2]);
    radio.flush_tx();
    radio.write( &mood, sizeof(mood) );

}

void SetGPRSMode(void) {                                            // Функция инициации GPRS в режим чтения СМС;
     Serial2.print("AT+CMGF=1\r");
     delay(300);
     Serial2.print("AT+IFC=1, 1\r");
     delay(300);
     Serial2.print("AT+CPBS=\"SM\"\r");
     delay(300);
     Serial2.print("AT+CNMI=1,2,2,1,0\r");
     delay(500);
     Serial2.println("AT+CMGD=1,4\r");
}

//************************ SendSMSString ************************//
void SendSMSString(String stringText) {                             // Отправка СМС сообщения;
    Serial2.print("AT+CMGF=1\r");
    delay(100);
    Serial2.println("AT + CMGS = \"+71111111111\"");                // На какой телефон;
    delay(100);
    Serial2.println(stringText);                                    // Текст сообщения.
    delay(100);
    Serial2.println((char)26);                                      // Отправляем Ctrl+Z, сообщение готово!
    return;
}

void SwitchAllOff (uint8_t what) {                                  // Функция выключения всего с двумя аргументами,
    if (what == 1) {                                                // Сначала включить все;
        SendNRFCommand(101);
        PWMWrite(255,255,255);
        digitalWrite(pinShelf, 1);
        delay(1500);
    }
    SendNRFCommand(100);                                            // Или выключить;
    PWMWrite(0,0,0);
    digitalWrite(pinShelf, 0);
    flag.switchOn = 0;
    return;
}

void TouchGPRS(void) {                                              // Каждые 5 секунд толкаем GPRS
    if (millis() >= timeToUpdateGPRS) {
        Serial2.println("AT");
 		timeToUpdateGPRS += 5000;
// 		Serial.println("Touch GPRS");								// Чтобы иногда смотреть что происходит;
// 		Serial.print("Mode of Unit is ");
//        Serial.println(modesOfUnit);
//        Serial.print("Flag.securityON is  ");
//        Serial.println(flag.securityOn);
//        Serial.print("timeWhenDesigionMade is  ");
//        Serial.println(timeWhenDesigionMade);
    }
}


void CheckGPRS(void) {

//      Serial.println("Now Write \"HIGH\" To Pin 9!");
            digitalWrite(9, HIGH);
            delay(1200);
//            Serial.println("Now Write \"LOW\" To Pin 9!");
            digitalWrite(9, LOW);
            delay(5000);
}

void DialVoiceCall()                                        // Функция звонка на номер;
{
//  Serial.println("Dial!!!");
  Serial2.println("ATD + +71111111111;");                   // Звонок на номер;
  delay(100);
  Serial2.println();

  delay(5000);
}

void PWMWrite (uint8_t red, uint8_t green, uint8_t blue) {  // Функция записи в каналы PWM;
    analogWrite(pinRed,   red);
    analogWrite(pinGreen, green);
    analogWrite(pinBlue,  blue);
}
